Package: iCARE
Title: Individualized Coherent Absolute Risk Estimation (iCARE)
Version: 1.39.0
Date: 2018-12-03
Author: Parichoy Pal Choudhury, Paige Maas, William Wheeler, Nilanjan Chatterjee
Description: An R package to build, validate and apply absolute risk models
Maintainer: Parichoy Pal Choudhury <Parichoy.PalChoudhury@cancer.org>
Depends: R (>= 3.3.0), plotrix, gtools, Hmisc
Suggests: RUnit, BiocGenerics
License: GPL-3 + file LICENSE
LazyData: true
biocViews: Software, StatisticalMethod, GenomeWideAssociation
NeedsCompilation: yes
Packaged: 2025-10-30 21:35:30 UTC; biocbuild
git_url: https://git.bioconductor.org/packages/iCARE
git_branch: devel
git_last_commit: 5d81f64
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-30
