import(stats, utils, graphics, plotrix, gtools, Hmisc)
useDynLib(iCARE)
export(computeAbsoluteRisk, computeAbsoluteRiskSplitInterval, ModelValidation, plotModelValidation)
S3method(print, icare)
S3method(summary, icare)
S3method(plot, icare)
S3method(print, icareSplit)
S3method(summary, icareSplit)
S3method(plot, icareSplit)
S3method(print, icareValid)
S3method(plot, icareValid)
